/*-
 * SPDX-License-Identifier: BSD-2-Clause
 * 
 * Copyright (c) 2021 Maarten L. Hekkelman, NKI-AVL
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <filesystem>

#include <zeep/nvp.hpp>

// --------------------------------------------------------------------

/// \brief Return the UniprotID and chunk number for an AlphaFold ID.
///
/// Split an id in the form of AF-UNIPROTID-F<CHUNKNR>
std::tuple<std::string,int> parse_af_id(std::string af_id);

// --------------------------------------------------------------------

struct compound
{
	std::string id;
	std::string analogue;
	// std::string name;
	uint32_t count_structures;
	uint32_t count_transplants;

	template<typename Archive>
	void serialize(Archive &ar, unsigned long)
	{
		ar & zeep::make_nvp("id", id)
		   & zeep::make_nvp("analogue", analogue)
		//    & zeep::make_nvp("name", name)
		   & zeep::make_nvp("structure-count", count_structures)
		   & zeep::make_nvp("transplant-count", count_transplants);
	}
};

struct structure
{
	std::string name;
	uint32_t count_hits;
	uint32_t count_transplants;
	uint32_t distinct_analogues;

	template<typename Archive>
	void serialize(Archive &ar, unsigned long)
	{
		ar & zeep::make_nvp("name", name)
		   & zeep::make_nvp("hit-count", count_hits)
		   & zeep::make_nvp("transplant-count", count_transplants)
		   & zeep::make_nvp("distinct-analogues", distinct_analogues);
	}
};

class data_service
{
  public:
	static data_service &instance();

	static int rebuild(const std::string &db_user, const std::filesystem::path &db_dir);

	std::vector<compound> get_compounds(float min_identity) const;
	std::vector<structure> get_structures(float min_identity, uint32_t page, uint32_t pageSize) const;
	std::vector<structure> get_structures_for_compound(float min_identity, const std::string &compound, uint32_t page, uint32_t pageSize) const;

	uint32_t count_structures(float min_identity) const;
	uint32_t count_structures(float min_identity, const std::string &compound) const;
};